@@ -1,31 +1,53 @@
-use ergo_fs::{expand, Path, PathDir, WalkDir};
-use std::{collections::HashMap, fs};
+use ergo_fs::{expand, Path, PathArc, WalkDir};
+use std::{
+    collections::HashMap,
+    fs::{self, create_dir_all},
+};
 use yaml_rust::Yaml;
 
 use crate::config::{validation_rules::required::Required, validator::validate_named_args};
 
-pub fn expand_dir(dir: &str, create: bool) -> Result<PathDir, String> {
-    let expanded_dir = expand(dir);
+pub fn is_file_path(path: &PathArc) -> bool {
+    return path
+        .components()
+        .last()
+        .unwrap()
+        .as_os_str()
+        .to_owned()
+        .into_string()
+        .unwrap()
+        .contains('.');
+}
 
-    if let Err(err_expand_dir) = expanded_dir {
-        return Err(err_expand_dir.to_string());
+fn create_missing_directories(path: &PathArc) -> Result<(), std::io::Error> {
+    if is_file_path(path) {
+        let parent = path.parent();
+
+        if parent.is_none() {
+            return Ok(());
+        }
+        return create_dir_all(parent.unwrap());
     }
 
-    let expanded_dir = expanded_dir.unwrap();
+    create_dir_all(path)
+}
+
+pub fn expand_path(path: &str, create: bool) -> Result<PathArc, String> {
+    let expanded_path = expand(path);
+
+    if let Err(err_expand_path) = expanded_path {
+        return Err(err_expand_path.to_string());
+    }
+    let expanded_path = PathArc::new(expanded_path.unwrap().to_string());
+
     if create {
-        let create_result = fs::create_dir_all(expanded_dir.to_string());
-        if let Err(err_create) = create_result {
-            return Err(err_create.to_string());
+        let create_result = create_missing_directories(&expanded_path);
+        if let Err(err_create_missing_directories) = create_result {
+            return Err(err_create_missing_directories.to_string());
         }
     }
 
-    let path = PathDir::new(expanded_dir.to_string());
-
-    if let Err(err_path) = path {
-        return Err(err_path.to_string());
-    }
-
-    Ok(path.unwrap())
+    Ok(expanded_path)
 }
 
 pub static DIR_SRC: &str = "src";
@@ -89,7 +111,7 @@ pub fn get_source_and_target(args: Yaml) -> Result<Dirs, String> {
 }
 
 // TODO: improve this with a better regex approach :)
-fn is_ignored(path: &Path, source: &PathDir, ignore: &[Yaml]) -> bool {
+fn is_ignored(path: &Path, source: &PathArc, ignore: &[Yaml]) -> bool {
     let path_str = path.strip_prefix(source).unwrap().to_str().unwrap();
 
     for ignore_path in ignore {
@@ -103,11 +125,40 @@ fn is_ignored(path: &Path, source: &PathDir, ignore: &[Yaml]) -> bool {
 }
 
 pub fn walk_files<O: Fn(&Path, &Path)>(
-    source: &PathDir,
+    source: &PathArc,
     target: &Path,
     ignore: Vec<Yaml>,
     op: O,
 ) -> Result<(), String> {
+    if !source.exists() {
+        return Err(format!(
+            "Source directory/file does not exist: {}",
+            source.to_string_lossy()
+        ));
+    }
+
+    if source.is_file() {
+        let source_file_name = source.file_name().unwrap().to_str().unwrap();
+        let source_file_ending = source_file_name.split('.').last().unwrap();
+
+        let target_file_ending = target
+            .file_name()
+            .unwrap()
+            .to_str()
+            .unwrap()
+            .split('.')
+            .last()
+            .unwrap();
+
+        if source_file_ending != target_file_ending {
+            op(source, target.join(source_file_name).as_path())
+        } else {
+            op(source, target)
+        }
+
+        return Ok(());
+    }
+
     for dir_entry in WalkDir::new(&source).min_depth(1) {
         let dir_entry = dir_entry.unwrap();
         let source_path = dir_entry.path();
@@ -139,9 +190,17 @@ pub fn walk_files<O: Fn(&Path, &Path)>(
 mod test {
     use super::*;
 
+    #[test]
+    fn it_fails_when_dir_doesnt_exist() {
+        let source = PathArc::new("/tmp/does_not_exist");
+        let target = PathArc::new("/tmp/target");
+
+        assert!(walk_files(&source, &target, vec![], |_, _| {}).is_err());
+    }
+
     #[test]
     fn it_expands_str_to_path() {
-        let expanded_dir = expand_dir("~", false);
+        let expanded_dir = expand_path("~", false);
 
         assert!(expanded_dir.is_ok());
         assert_eq!(
@@ -150,19 +209,29 @@ mod test {
         );
     }
 
+    #[test]
+    fn it_returns_true_if_the_path_is_a_file() {
+        assert!(is_file_path(&PathArc::new("/tmp/test.txt")));
+    }
+
+    #[test]
+    fn it_returns_false_if_the_path_is_a_directory() {
+        assert!(!is_file_path(&PathArc::new("/tmp/test")));
+    }
+
     #[test]
     fn it_creates_intermediate_dirs_when_needed() {
         let temp_dir = tempfile::tempdir().unwrap();
         let temp_dir_path = temp_dir.path();
-        let temp_dir_path_complete = temp_dir_path.join("test");
+        let target_dir = temp_dir_path.join("test");
 
-        let expanded_dir = expand_dir(temp_dir_path_complete.to_str().unwrap(), true);
+        let expanded_dir = expand_path(target_dir.to_str().unwrap(), true);
 
         assert!(expanded_dir.is_ok());
         assert_eq!(
             expanded_dir.unwrap().to_string_lossy(),
-            temp_dir_path_complete.to_string_lossy()
+            target_dir.to_string_lossy()
         );
-        assert!(temp_dir_path_complete.exists());
+        assert!(target_dir.exists());
     }
 }
